-- domstat.sql
--
-- procedure domstat(domids, aid)
-- domids: set of domain IDs
-- aid: accessID
--
-- update table domstat (grouped monitoring states per domain)

-- Status-Tabellen der Limits updaten
--
--
-- Beteiligte Tabellen
--
--  domstat       Liste der letzten Aktualisierungen der Limitgruppen Stati
--    verfahrenID:   ID des Verfahrens
--    accessID:      AccessID
--    TS:            Zeitpunkt der Aktualisierung
--    workerid:      connection_id der Verbindung bei Aktualisierung, sonst 0
--  Werte des Verfahrens domainID werden maximal alle 3 Minuten aktualisiert.
--  Wenn workerid für mehr als 6 Minunte gesetzt ist, dann wird trotzdem aktualisiert.
--
--
--  domstats             Stati pro serverID, sectionID, teststepID, teststepdetailsID, accessID
--    serverID:          Server ID
--    sectionID:         Section ID
--    teststepID:        Teststep ID
--    teststepdetailsID: Detail ID
--    accessID:          Access ID
--    verfahrenID:       ID des Verfahrens
--    numlnone:          Limit Anzahl None
--    numlinfo:          Limit Anzahl Info
--    numlwarn:          Limit Anzahl Warn
--    numlerror:         Limit Anzahl Error
--    numinone:          isAlive Anzahl None
--    numiinfo:          isAlive Anzahl Info
--    numiwarn:          isAlive Anzahl Warn
--    numierror:         isAlive Anzahl Error
--    numcnone:          Kombilimit Anzahl None
--    numcinfo:          Kombilimit Anzahl Info
--    numcwarn:          Kombilimit Anzahl Warn
--    numcerror:         Kombilimit Anzahl Error
--    numclnone:         komprimiertes Limit Anzahl None
--    numclinfo:         komprimiertes Limit Anzahl Info
--    numclwarn:         komprimiertes Limit Anzahl Warn
--    numclerror:        komprimiertes Limit Anzahl Error
--    numccnone:         komprimiertes Kombilimit Anzahl None
--    numccinfo:         komprimiertes Kombilimit Anzahl Info
--    numccwarn:         komprimiertes Kombilimit Anzahl Warn
--    numccerror:        komprimiertes Kombilimit Anzahl Error
--

set @@character_set_client = 'utf8';

delimiter //

DROP procedure IF EXISTS domstat//
create procedure domstat(domids varchar(20000), aid integer unsigned)
  deterministic
  sql security invoker
 begin

  -- loggingtmp.domids  Verfahren bestimmen, die upgedatet werden muessen
  --   newdom  0: muss upgedatet werden
  --           1: noch nicht in domstat vorhanden
  create temporary table loggingtmp.domids (primary key(verfahrenID))
    select ID as verfahrenID, TS is NULL as newdom
    from verfahren
    left join domstat on domstat.verfahrenID = verfahren.ID and accessID = aid
    where find_in_set(verfahren.ID, domids)
      and (TS is NULL OR (workerid = 0 AND UTC_TIMESTAMP() > TS + interval 3 minute)
           OR UTC_TIMESTAMP() > TS + interval 10 minute);

  if row_count() > 0 then

    update domstat
      join loggingtmp.domids on domids.verfahrenID = domstat.verfahrenID and not domids.newdom
        set workerid = CONNECTION_ID(), TS = UTC_TIMESTAMP()
      where accessID = aid;

    insert ignore into domstat (verfahrenID, workerid, TS)
      select verfahrenID, CONNECTION_ID(), UTC_TIMESTAMP()
      from loggingtmp.domids where newdom;

    create temporary table loggingtmp.lims (primary key (werteindizesID))
      select distinct werteindizesID
      from wertelimits
      where aktiv = '1' AND vergleich != '';
    insert ignore into loggingtmp.lims (werteindizesID)
      select distinct werteindizesID
      from isalive
      where aktiv = '1';

    create temporary table loggingtmp.up (key verfahrenID(verfahrenID))
      select verfahrenID, server, server_v, classes, classes_v, section, section_v,
        teststep, teststep_v, details, details_v, einheit, einheit_v
      from ueberwachungspausen
      join loggingtmp.domids using(verfahrenID)
      where datumvon <= now() and now() <= datumbis
          and find_in_set(weekday(curdate()), weekdays)
          and zeitvon <= curtime() and curtime() <= zeitbis
          and (mdays = '' OR find_in_set(day(curdate()), mdays))
          and accessID = aid;

    create temporary table loggingtmp.wids
          (primary key (werteindizesID))
      select w.ID as werteindizesID,
        w.serverID, w.sectionID, w.teststepID, w.teststepdetailsID, s.verfahrenID
      from werteindizes as w
      join loggingtmp.lims on lims.werteindizesID = w.ID
      join server as s on s.ID = w.serverID and s.state = 'active'
      join loggingtmp.domids using(verfahrenID)
      join section as e on e.ID = w.sectionID
      join teststep as t on t.ID = w.teststepID
      join teststepdetails as d on d.ID = w.teststepdetailsID
      join einheiten as u on u.ID = d.einheitID
      left join serverclasses as c on c.ID = s.serverclassesID
      left join loggingtmp.up
        on up.verfahrenID = s.verfahrenID
          and testreg(coalesce(c.name, ''), up.classes, up.classes_v)
          and testreg(s.servername, up.server, up.server_v)
          and testreg(e.name, up.section, up.section_v)
          and testreg(t.name, up.teststep, up.teststep_v)
          and testreg(d.name, up.details, up.details_v)
          and testreg(u.name, up.einheit, up.einheit_v)
      where w.accessID = aid and up.verfahrenID is NULL;

    create temporary table loggingtmp.limstats
        (primary key (werteindizesID))
      select wl.werteindizesID, wids.verfahrenID,
        sum(twl.msglevel = 'none') as numok,
        sum(twl.msglevel = 'info') as numinfo,
        sum(twl.msglevel = 'warn') as numwarn,
        sum(twl.msglevel = 'error') as numerror
      from wertelimits as wl
      join twertelimits as twl on twl.ID = wl.ID
      join loggingtmp.wids using(werteindizesID)
      left join autolimits as da on da.ID = wl.autolimitsID and da.aktiv = '0'
      left join gautolimits as ga on ga.ID = wl.groupautoID and ga.aktiv = '0'
      where wl.aktiv = '1' and wl.vergleich != '' and not wl.hidden and da.ID is NULL and ga.ID is NULL and msglevel != 'unknown'
      group by wl.werteindizesID;

    create temporary table loggingtmp.isastats
        (primary key (werteindizesID))
      select il.werteindizesID, wids.verfahrenID,
        sum(msgdate <= lastdatum) as numok,
        sum(il.msglevel = 'info' and msgdate > lastdatum) as numinfo,
        sum(il.msglevel = 'warn' and msgdate > lastdatum) as numwarn,
        sum(il.msglevel = 'error' and msgdate > lastdatum) as numerror
      from isalive as il
      join tisalive as til on til.ID = il.ID
      join loggingtmp.wids using(werteindizesID)
      join letztewerte as lw on lw.ID = wids.werteindizesID
      left join autoisalive as da on da.ID = il.autoisaliveID and da.aktiv = '0'
      left join gautoisalive as ga on ga.ID = il.groupautoID and ga.aktiv = '0'
      where il.aktiv = '1' and not il.hidden and da.ID is NULL and ga.ID is NULL
      group by il.werteindizesID;

    create temporary table loggingtmp.climstats
        (primary key (werteindizesID))
      select w.ID as werteindizesID, s.verfahrenID,
        sum(twl.msglevel = 'none') as numok,
        sum(twl.msglevel = 'info') as numinfo,
        sum(twl.msglevel = 'warn') as numwarn,
        sum(twl.msglevel = 'error') as numerror
      from lwertelimits as wl
      join tlwertelimits as twl on twl.ID = wl.ID
      join lwerteindizes as lw on lw.ID = wl.lwerteindizesID and lw.accessID = aid
      join server as s on s.ID = lw.serverID and s.state = 'active'
      join loggingtmp.domids using(verfahrenID)
      join section as e on e.ID = lw.sectionID
      join teststep as t on t.ID = lw.teststepID
      join teststepdetails as d on d.ID = lw.teststepdetailsID
      join einheiten as u on u.ID = d.einheitID
      join werteindizes as w using(serverID, sectionID, teststepID, teststepdetailsID, accessID)
      join groupfunctions as gf on gf.ID = lw.groupfunctionsID
      left join serverclasses as c on c.ID = s.serverclassesID
      left join autollimits as da on da.ID = wl.autollimitsID and da.aktiv = '0'
      left join gautoclimits as ga on ga.ID = wl.groupautoID and ga.aktiv = '0'
      left join loggingtmp.up
        on up.verfahrenID = s.verfahrenID
          and testreg(coalesce(c.name, ''), up.classes, up.classes_v)
          and testreg(s.servername, up.server, up.server_v)
          and testreg(e.name, up.section, up.section_v)
          and testreg(t.name, up.teststep, up.teststep_v)
          and testreg(concat(gf.code, '[', left(period,1), '](', d.name, ')'), up.details, up.details_v)
          and testreg(u.name, up.einheit, up.einheit_v)
      where wl.aktiv = '1' and wl.vergleich != '' and not wl.hidden and da.ID is NULL and ga.ID is NULL and up.verfahrenID is NULL and msglevel != 'unknown'
      group by w.ID;

    create temporary table loggingtmp.widstats (
      werteindizesID int unsigned not null default '0',
      verfahrenID int unsigned not null default '0',
      numlok     int unsigned not null default '0',
      numlinfo   int unsigned not null default '0',
      numlwarn   int unsigned not null default '0',
      numlerror  int unsigned not null default '0',
      numiok     int unsigned not null default '0',
      numiinfo   int unsigned not null default '0',
      numiwarn   int unsigned not null default '0',
      numierror  int unsigned not null default '0',
      numclok    int unsigned not null default '0',
      numclinfo  int unsigned not null default '0',
      numclwarn  int unsigned not null default '0',
      numclerror int unsigned not null default '0',
      primary key(werteindizesID));

    insert into loggingtmp.widstats (werteindizesID, verfahrenID)
      select werteindizesID, verfahrenID from loggingtmp.limstats
      union distinct select werteindizesID, verfahrenID from loggingtmp.isastats
      union distinct select werteindizesID, verfahrenID from loggingtmp.climstats;

    update loggingtmp.widstats
      join (select werteindizesID, sum(numok) as nok, sum(numinfo) as ninfo, sum(numwarn) as nwarn, sum(numerror) as nerror
            from loggingtmp.limstats
            group by werteindizesID
            ) as j using(werteindizesID)
        set numlok = nok, numlinfo = ninfo, numlwarn = nwarn, numlerror = nerror;
    update loggingtmp.widstats
      join (select werteindizesID, sum(numok) as nok, sum(numinfo) as ninfo, sum(numwarn) as nwarn, sum(numerror) as nerror
            from loggingtmp.isastats
            group by werteindizesID
            ) as j using(werteindizesID)
        set numiok = nok, numiinfo = ninfo, numiwarn = nwarn, numierror = nerror;
    update loggingtmp.widstats
      join (select werteindizesID, sum(numok) as nok, sum(numinfo) as ninfo, sum(numwarn) as nwarn, sum(numerror) as nerror
            from loggingtmp.climstats
            group by werteindizesID
            ) as j using(werteindizesID)
        set numclok = nok, numclinfo = ninfo, numclwarn = nwarn, numclerror = nerror;

    create temporary table loggingtmp.combistats
        (primary key (serverID, sectionID, teststepID))
      select cmbl.serverID, cmbl.sectionID, cmbl.teststepID, s.verfahrenID,
        sum(up.verfahrenID is NULL and tcmbl.msglevel = 'none') as numok,
        sum(up.verfahrenID is NULL and tcmbl.msglevel = 'info') as numinfo,
        sum(up.verfahrenID is NULL and tcmbl.msglevel = 'warn') as numwarn,
        sum(up.verfahrenID is NULL and tcmbl.msglevel = 'error') as numerror
      from kombilimits as cmbl
      join tkombilimits as tcmbl on tcmbl.ID = cmbl.ID
      join server as s on s.ID = cmbl.serverID and s.state = 'active'
      join loggingtmp.domids using(verfahrenID)
      join section as e on e.ID = cmbl.sectionID
      join teststep as t on t.ID = cmbl.teststepID
      left join serverclasses as c on c.ID = s.serverclassesID
      left join autokombilimits as da on da.ID = cmbl.autokombilimitsID and da.aktiv = '0'
      left join gautocombilimits as ga on ga.ID = cmbl.groupautoID and ga.aktiv = '0'
      left join loggingtmp.up
        on up.verfahrenID = s.verfahrenID
          and testreg(coalesce(c.name, ''), up.classes, up.classes_v)
          and testreg(s.servername, up.server, up.server_v)
          and testreg(e.name, up.section, up.section_v)
          and testreg(t.name, up.teststep, up.teststep_v)
      where cmbl.aktiv = '1' and cmbl.relation != '' and not cmbl.hidden and cmbl.accessID = aid
        and da.ID is NULL and ga.ID is NULL and up.verfahrenID is NULL and tcmbl.msglevel != 'unknown'
      group by serverID, sectionID, teststepID;

    create temporary table loggingtmp.ccstats
        (primary key (serverID, sectionID, teststepID))
      select cmbl.serverID, cmbl.sectionID, cmbl.teststepID, s.verfahrenID,
        sum(up.verfahrenID is NULL and tcmbl.msglevel = 'none') as numok,
        sum(up.verfahrenID is NULL and tcmbl.msglevel = 'info') as numinfo,
        sum(up.verfahrenID is NULL and tcmbl.msglevel = 'warn') as numwarn,
        sum(up.verfahrenID is NULL and tcmbl.msglevel = 'error') as numerror
      from cclimits as cmbl
      join tcclimits as tcmbl on tcmbl.ID = cmbl.ID
      join server as s on s.ID = cmbl.serverID and s.state = 'active'
      join loggingtmp.domids using(verfahrenID)
      join section as e on e.ID = cmbl.sectionID
      join teststep as t on t.ID = cmbl.teststepID
      left join serverclasses as c on c.ID = s.serverclassesID
      left join autocclimits as da on da.ID = cmbl.domautoID and not da.aktiv
      left join gautocclimits as ga on ga.ID = cmbl.groupautoID and not ga.aktiv
      left join loggingtmp.up
        on up.verfahrenID = s.verfahrenID
          and testreg(coalesce(c.name, ''), up.classes, up.classes_v)
          and testreg(s.servername, up.server, up.server_v)
          and testreg(e.name, up.section, up.section_v)
          and testreg(t.name, up.teststep, up.teststep_v)
      where cmbl.aktiv and cmbl.relation != '' and not cmbl.hidden and cmbl.accessID = aid
        and da.ID is NULL and ga.ID is NULL and up.verfahrenID is NULL and tcmbl.msglevel != 'unknown'
      group by serverID, sectionID, teststepID;

    delete domstats from domstats join loggingtmp.domids using(verfahrenID)
      where accessID = aid;

    insert ignore into domstats
      (serverID, sectionID, teststepID, teststepdetailsID, accessID, verfahrenID,
       numlok, numlinfo, numlwarn, numlerror, numiok, numiinfo, numiwarn, numierror,
       numclok, numclinfo, numclwarn, numclerror)
      select serverID, sectionID, teststepID, teststepdetailsID, aid, wids.verfahrenID,
        numlok, numlinfo, numlwarn, numlerror, numiok, numiinfo, numiwarn, numierror,
        numclok, numclinfo, numclwarn, numclerror
      from loggingtmp.widstats
      join loggingtmp.wids using(werteindizesID)
      order by werteindizesID;

    insert into domstats
      (serverID, sectionID, teststepID, teststepdetailsID, accessID, verfahrenID,
       numcok, numcinfo, numcwarn, numcerror, numccok, numccinfo, numccwarn, numccerror)
      select c.serverID, c.sectionID, c.teststepID, 0, aid, c.verfahrenID,
        c.numok, c.numinfo, c.numwarn, c.numerror,
        coalesce(cc.numok, 0), coalesce(cc.numinfo, 0), coalesce(cc.numwarn, 0), coalesce(cc.numerror, 0)
      from loggingtmp.combistats as c
      left join loggingtmp.ccstats as cc using(serverID, sectionID, teststepID);
    insert into domstats
      (serverID, sectionID, teststepID, teststepdetailsID, accessID, verfahrenID,
       numcok, numcinfo, numcwarn, numcerror, numccok, numccinfo, numccwarn, numccerror)
      select cc.serverID, cc.sectionID, cc.teststepID, 0, aid, cc.verfahrenID,
        0, 0, 0, 0, cc.numok, cc.numinfo, cc.numwarn, cc.numerror
      from loggingtmp.ccstats as cc
      left join loggingtmp.combistats as c using(serverID, sectionID, teststepID)
      where c.serverID is NULL;

    drop temporary table loggingtmp.limstats, loggingtmp.isastats, loggingtmp.climstats, loggingtmp.widstats,
      loggingtmp.wids, loggingtmp.lims, loggingtmp.combistats, loggingtmp.ccstats, loggingtmp.up;

    update domstat join loggingtmp.domids using(verfahrenID)
      set workerid = 0, TS = UTC_TIMESTAMP()
      where workerid = CONNECTION_ID();
  end if;

  drop temporary table loggingtmp.domids;

  if exists (select * from domstat where workerid > 0 and find_in_set(verfahrenID, domids)) then
      select sleep(11) into @dummy;
    if exists (select * from domstat where workerid > 0 and find_in_set(verfahrenID, domids)) then
        select sleep(17) into @dummy;
      if exists (select * from domstat where workerid > 0 and find_in_set(verfahrenID, domids)) then
          select sleep(17) into @dummy;
      end if;
    end if;
  end if;

end//
